
import java.awt.event.*; 

import javax.media.opengl.*;
import javax.media.opengl.glu.*;
import com.sun.opengl.util.*;

public class Renderer extends MouseAdapter implements GLEventListener, KeyListener
{
	// Atributos
	private GL gl;
	private GLU glu;
	private GLUT glut;
	private GLAutoDrawable glDrawable;
	private double angle, fAspect;

	/**
	 * Construtor da classe Renderer que no recebe parmetros.
	 */
	public Renderer()
	{
		// Especifica o ngulo da projeo perspectiva  
		angle=44;   
		// Inicializa o valor para correo de aspecto   
		fAspect = 1;    
	}

	/**
	 * Mtodo definido na interface GLEventListener e chamado pelo objeto no qual ser feito o desenho
	 * logo aps a inicializao do contexto OpenGL. 
	 */    
	public void init(GLAutoDrawable drawable)
	{
		glDrawable = drawable;
		gl = drawable.getGL();      
		glu = new GLU();
		glut = new GLUT();

		drawable.setGL(new DebugGL(gl));        

		gl.glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
		gl.glEnable(GL.GL_DEPTH_TEST);         
	}

    public void desenhaChao()
    {
        gl.glColor3f(1,0,1);
        gl.glLineWidth(3);
        gl.glBegin(GL.GL_LINES);
        for(float z=-1000; z<=1000; z+=10)
        {
            gl.glVertex3f(-1000,-0.1f,z);
            gl.glVertex3f( 1000,-0.1f,z);
        }
        for(float x=-1000; x<=1000; x+=10)
        {
            gl.glVertex3f(x,-0.1f,-1000);
            gl.glVertex3f(x,-0.1f,1000);
        }
        gl.glEnd();
        gl.glLineWidth(1);
    }
    
	/**
	 * Mtodo definido na interface GLEventListener e chamado pelo objeto no qual ser feito o desenho
	 * para comear a fazer o desenho OpenGL pelo cliente.
	 */  
	public void display(GLAutoDrawable drawable)
	{
		gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT );
		gl.glLoadIdentity();    

		especificaParametrosVisualizacao(); 

		gl.glColor3f(0.0f, 0.0f, 1.0f);

		glut.glutWireTeapot(35); 
		gl.glTranslatef(0,-28,0);
		desenhaChao();
	}

	/**
	 * Mtodo definido na interface GLEventListener e chamado pelo objeto no qual ser feito o desenho
	 * depois que a janela foi redimensionada.
	 */  
	public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height)
	{
		gl.glViewport(0, 0, width, height);
		fAspect = (float)width/(float)height;      
	}

	/**
	 * Mtodo definido na interface GLEventListener e chamado pelo objeto no qual ser feito o desenho
	 * quando o modo de exibio ou o dispositivo de exibio associado foi alterado.
	 */  
	public void displayChanged(GLAutoDrawable drawable, boolean modeChanged, boolean deviceChanged) { }

	/**
	 * Mtodo usado para especificar a posio do observador virtual.
	 */    
	public void posicionaObservador()
	{
		// Especifica sistema de coordenadas do modelo
		gl.glMatrixMode(GL.GL_MODELVIEW);
		// Inicializa sistema de coordenadas do modelo
		gl.glLoadIdentity();
		// Especifica posio do observador e do alvo
	    glu.gluLookAt(0.0,80.0,200.0, 0.0,0.0,0.0, 0.0,1.0,0.0);   
	}
   
	/**
	 * Mtodo usado para especificar o volume de visualizao.
	 */    
	public void especificaParametrosVisualizacao()
	{
		// Especifica sistema de coordenadas de projeo
		gl.glMatrixMode(GL.GL_PROJECTION);
		// Inicializa sistema de coordenadas de projeo
		gl.glLoadIdentity();

		// Especifica a projeo perspectiva(angulo,aspecto,zMin,zMax)
		glu.gluPerspective(angle, fAspect, 0.5, 500);

		posicionaObservador();
	}

	/**
	 * Mtodo da classe MouseAdapter que est sendo sobrescrito para gerenciar os 
	 * eventos de clique de mouse, de maneira que sej feito zoom in e zoom out.
	 */  
	public void mouseClicked(MouseEvent e)
	{
		if (e.getButton() == MouseEvent.BUTTON1) // Zoom in
		if (angle >= 4) angle -= 4;
		if (e.getButton() == MouseEvent.BUTTON3) // Zoom out
		if (angle <= 72) angle += 4;
		glDrawable.display();
	}

	/**
	 * Mtodo definido na interface KeyListener que est sendo implementado para, 
	 * de acordo com as teclas pressionadas, permitir mover a posio do observador
	 * virtual.
	 */        
	public void keyPressed(KeyEvent e)
	{
		switch (e.getKeyCode())
		{ 
			case KeyEvent.VK_ESCAPE:	System.exit(0);
										break; 										
		}  
		glDrawable.display();
	}

	/**
	 * Mtodo definido na interface KeyListener.
	 */      
	public void keyTyped(KeyEvent e) { }

	/**
	 * Mtodo definido na interface KeyListener.
	 */       
	public void keyReleased(KeyEvent e) { }
}

